library responsive_textview;

import 'package:flutter/material.dart';

import 'utils/screen/screen_util.dart';

class ResponsiveTextView extends StatelessWidget {
  // The default font size if none is specified.
  static const double _defaultFontSize = 14.0;

  // Single instance
  static ScreenUtil _screenUtil;

  final String text;
  final Key key;
  final TextSpan textSpan;
  final TextStyle style;
  final StrutStyle strutStyle;
  final TextAlign textAlign;
  final TextDirection textDirection;
  final Locale locale;
  final bool softWrap;
  final TextOverflow overflow;
  final double textScaleFactor;
  final int maxLines;
  final String semanticsLabel;

  // If this is not provided then the same font size (provided by the user in styles)
  // will be applied in landscape mode in sp
  final double fontSizeInLandscape;

  // If this is not provided then the same font size (provided by the user in styles)
  // will be applied in portrait mode in sp
  final double fontSizeInPortrait;

  // If this is not provided then the same color (provided by the user in styles)
  // will be applied in light mode
  final Color colorInLightMode;

  // If this is not provided then the default color (provided by the user in styles)
  // will be applied in dark mode
  final Color colorInDarkMode;

  // Enabling this will allow to see the font size and orientation used by
  // ResponsiveTextView
  final bool isDebugEnabled;

  ResponsiveTextView(
    this.text, {
    this.key,
    this.style,
    this.strutStyle,
    this.textAlign,
    this.textDirection,
    this.locale,
    this.softWrap,
    this.overflow,
    this.textScaleFactor,
    this.maxLines,
    this.semanticsLabel,
    this.fontSizeInLandscape,
    this.fontSizeInPortrait,
    this.colorInLightMode,
    this.colorInDarkMode,
    this.isDebugEnabled = false,
  })  : assert(
          text != null,
          'A non-null String must be provided to a ResponsiveTextView widget.',
        ),
        textSpan = null,
//        assert(color == null || foreground == null, _kColorForegroundWarning),

        super(key: key);

  @override
  Widget build(BuildContext context) {
    _screenUtil = ScreenUtil.instance..init(context);
    return Text(
      text,
      key: key,
      style: getTextStyle(context),
      strutStyle: strutStyle,
      textAlign: textAlign,
      textDirection: textDirection,
      locale: locale,
      softWrap: softWrap,
      overflow: overflow,
      textScaleFactor: textScaleFactor,
      maxLines: maxLines,
      semanticsLabel: semanticsLabel,
    );
  }

  //general methods:------------------------------------------------------------
  TextStyle getTextStyle(BuildContext context) {
    TextStyle textStyle;

    // check to see if style is null, then create a default style in-order to
    // apply user provided colors (dark/light mode) and text sizes for landscape/portrait
    if (style == null) {
      textStyle =
          TextStyle(fontSize: _getFontSize, color: _getTextColor(context));
    } else {
      textStyle = style.copyWith(
        fontSize: _getFontSize,
        color: _getTextColor(context),
      );
    }

    // return style object with new configurations
    return textStyle;
  }

  double get _getFontSize {
    double fontSize = _defaultFontSize;

    // check to see if either of the landscape and portrait font is provided, then
    // apply the provided font according to the device orientation, else, use
    // the default font
    if (ScreenUtil.mediaQueryData.orientation == Orientation.portrait &&
        fontSizeInPortrait != null) {
      fontSize = fontSizeInPortrait;
    } else if (ScreenUtil.mediaQueryData.orientation == Orientation.landscape &&
        fontSizeInLandscape != null) {
      fontSize = fontSizeInLandscape;
    } else if (style != null && style.fontSize != null) {
      fontSize = style.fontSize;
    }

    // this block will only execute when isDebugEnabled == true
    if (isDebugEnabled) {
      print('Font size: $fontSize');
      print('Orientation: ${ScreenUtil.mediaQueryData.orientation}');
    }

    return _screenUtil.setSp(fontSize);
  }

  Color _getTextColor(BuildContext context) {
    Color textColor;

    // check to see if user provided text color either in light or dark mode
    // if yes, then apply color accordingly, else apply the default color
    if (Theme.of(context).brightness == Brightness.dark &&
        colorInDarkMode != null) {
      textColor = colorInDarkMode;
    } else if (Theme.of(context).brightness == Brightness.light &&
        colorInLightMode != null) {
      textColor = colorInLightMode;
    }

    // this block will only execute when isDebugEnabled == true
    if (isDebugEnabled) {
      print('Text color: $textColor');
      print(
          'Theme mode: ${Theme.of(context).brightness == Brightness.dark ? Brightness.dark : Brightness.light}');
    }

    return textColor;
  }
}
